/* 
 * Copyright (c) 2011, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * */
/*
 *  ======== simpleTask.c ========
 *  This example exercises just runs a task every millisecond
 */

 /*  ----------------------------------- XDC module Headers           */
#include <xdc/std.h>

#include <xdc/runtime/Log.h>
#include <xdc/runtime/Types.h>
#include <xdc/runtime/Diags.h>
#include <xdc/runtime/System.h>

/*  ----------------------------------- BIOS module Headers           */
#include <ti/sysbios/BIOS.h>
#include <ti/sysbios/hal/Timer.h>
#include <ti/sysbios/knl/Task.h>

/*  ----------------------------------- IPC module Headers           */
#include <ti/ipc/Ipc.h>
#include <ti/ipc/MultiProc.h>

/*  ----------------------------------- UIA module Headers           */
#include <ti/uia/events/UIABenchmark.h>

#include <xdc/cfg/global.h>

Char buffer[128];

/*
 *  ======== main ========
 */
Void main()
{
    Int status = Ipc_S_SUCCESS;
    
    Log_print0(Diags_USER1, "stairstep example started.");    
        
    /* Call Ipc_start() */
    Ipc_start();
    
    do {
        status = Ipc_attach(MultiProc_getId("HOST"));
    } while (status < 0);

    BIOS_start();
    
    return;                 
}

/*
 *  ======== reverseBits ========
 *  Send and receive messages
 */
Void reverseBits(Char *buf, SizeT len)
{
    Int  i;
    Int  j;    
    Char h;    
    
    for (i = 0; i < len; i++) {
        
        for(h = j = 0; j < 8; j++) {
            h = (h << 1) + (buf[i] & 1); 
            buf[i] >>= 1; 
        }
    }

    return;
}

/*
 *  ======== taskLoad ========
 */
Void task(UArg arg1, UArg arg2)
{   
    UInt count = 1;
    
    while (TRUE) {        
        Semaphore_pend(sem, BIOS_WAIT_FOREVER);        
        
        /* Benchmark how long it takes to flip all the bits */
        Log_write1(UIABenchmark_start, (xdc_IArg)"Reverse");
        reverseBits(buffer, sizeof(buffer));
        Log_write1(UIABenchmark_stop, (xdc_IArg)"Reverse");
        
        Log_print1(Diags_USER1, "count = %d", count++);        
    }
}

/*
 *  ======== clockFxn ========
 *  Runs every N ms (look in the simpleTask.cfg.xs file).
 */
Void clockFxn(UArg arg)
{
    /* Kick of Task thread */
    Semaphore_post(sem);
}
/*
 */
/*
 *  @(#) ti.uia.examples.evmti816x; 2, 0, 0, 0,5; 6-15-2011 11:42:24; /db/vtree/library/trees/uia/uia.git/src/ null
 */

