/* 
 * Copyright (c) 2011, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * */
 
/*
 * ======== MCSA Tutorial 1D : Working with UIARoundtrip events and .cfg files ========
 * This tutorial demonstrates the steps needed to configure an
 * a UIA module for use with an application.  It also demonstrates 
 * the use of the UIARoundtrip events.
 *
 * See http://processors.wiki.ti.com/index.php/McsaTutorial1 for more info.
 */

#include <xdc/std.h>
#include <xdc/runtime/Log.h>

#define IS_TUTORIAL_D_ENABLED (FALSE)

#if IS_TUTORIAL_D_ENABLED
/* Including the following .h file would seem to be sufficient to allow the C
 * code to use the events defined in the UIARoundtrip module.  
 * If you try to compile this file, however, it will fail unless the UIARoundtrip module 
 * has been configured for use in the project's .cfg file!
 */
#include <ti/uia/events/UIARoundtrip.h>
#endif

void tutorial_1D();

void tutorial_1D(){

#if IS_TUTORIAL_D_ENABLED
         /*  The UIARoundtrip module defines events that allow
         *  tooling to analyze the absolute time elapsed between
         *  a start point event and a stop point event.  It differs
         *  from UIABenchmark in that UIABenchmark supports context-awareness
         *  (i.e. exclusive time elapsed) whereas UIARoundtrip is always
         *  inclusive (i.e. the time elapsed that DVT displays includes
         *  time spent in all other threads / tasks).
         */
        Log_write3(UIARoundtrip_startInstanceWithStr,(IArg)"Roundtrip Log_write3",4,(IArg)"Roundtrip");
        /* Often, the UIARoundtrip_stopInstanceWithStr event will be logged
         * by a different task than that used to log the UIARoundtrip_startInstanceWithStr
         * event.  It can even be logged by a different CPU to measure the time 
         * it takes for a multicore application to complete some coordinated activity.
         * This topic will be covered in more detail in the tutorial on Multicore Event Correlation. 
         */
        Log_write3(UIARoundtrip_stopInstanceWithStr,(IArg)"Roundtrip Log_write3",4,(IArg)"Roundtrip");

#else
        Log_warning0("IS_TUTORIAL_D_ENABLED == FALSE.");
#endif

}
/*
 *  @(#) ti.uia.examples.tutorial1; 1, 0, 0, 0,4; 6-15-2011 11:42:34; /db/vtree/library/trees/uia/uia.git/src/ null
 */

