/* 
 * Copyright (c) 2011, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * */
/*
 *  ======== TimestampC64X.c ========
 */

#include <xdc/std.h>
#include <xdc/runtime/Types.h>
#include <xdc/runtime/Startup.h>
#include <xdc/runtime/Gate.h>
#include "package/internal/TimestampDM816XTimer.xdc.h"

#define TIMER_TCLR_START_ONESHOT        0x1
#define TIMER_TCLR_START_CONTINUOUS     0x3
#define TIMER_TCLR_STOP_MASK            0xfffffffe
#define TIMER_TWPS_W_PEND_TMAR          0x10
#define TIMER_TWPS_W_PEND_TLDR          0x4
#define TIMER_TWPS_W_PEND_TCRR          0x2
#define TIMER_TWPS_W_PEND_TCLR          0x1
#define TIMER_TMAR_MATCH_VALUE          0x0

typedef volatile struct TimerRegs {
    UInt tidr;
    UInt empty[3];
    UInt tiocpCfg;
    UInt empty1[3];
    UInt irq_eoi;
    UInt irqstat_raw;
    UInt tisr;  /* irqstat */
    UInt tier;  /* irqen_set */
    UInt irqen_clr;
    UInt twer;  /* irqwaken; */
    UInt tclr;
    UInt tcrr;
    UInt tldr;
    UInt ttgr;
    UInt twps;
    UInt tmar;
    UInt tcar1;
    UInt tsicr;
    UInt tcar2;
} TimerRegs;
    
Int TimestampDM816XTimer_Module_startup(Int state) {

    TimerRegs *timer = NULL;
    Int* pPrcmTimerClkCtrlReg = NULL;
    IArg key;
    module->timerMSW = 0;
    module->timerBaseAdrs = NULL;

        if (TimestampDM816XTimer_timerBaseAdrs == NULL){
#ifndef xdc_target__isaCompatible_64P
                  switch(TimestampDM816XTimer_timerNumber){
                  case TimestampDM816XTimer_TimerInstance_Timer4:
                          module->timerBaseAdrs = (Ptr)TimestampDM816XTimer_TimerBaseAdrs_C6000_Timer4;
                          break;
                  case TimestampDM816XTimer_TimerInstance_Timer5:
                          module->timerBaseAdrs = (Ptr)TimestampDM816XTimer_TimerBaseAdrs_C6000_Timer5;
                          break;
                  case TimestampDM816XTimer_TimerInstance_Timer6:
                          module->timerBaseAdrs = (Ptr)TimestampDM816XTimer_TimerBaseAdrs_C6000_Timer6;
                          break;
                  case TimestampDM816XTimer_TimerInstance_Timer7:
                          module->timerBaseAdrs = (Ptr)TimestampDM816XTimer_TimerBaseAdrs_C6000_Timer7;
                          break;
                  default:
                          break;
                  }
#else
                  /* both CortexA8 and CortexM3 have the same base addresses for the timers */
                  switch(TimestampDM816XTimer_timerNumber){
                  case TimestampDM816XTimer_TimerInstance_Timer4:
                          module->timerBaseAdrs = (Ptr)TimestampDM816XTimer_TimerBaseAdrs_ARM_Timer4;
                          break;
                  case TimestampDM816XTimer_TimerInstance_Timer5:
                          module->timerBaseAdrs = (Ptr)TimestampDM816XTimer_TimerBaseAdrs_ARM_Timer5;
                          break;
                  case TimestampDM816XTimer_TimerInstance_Timer6:
                          module->timerBaseAdrs = (Ptr)TimestampDM816XTimer_TimerBaseAdrs_ARM_Timer6;
                          break;
                  case TimestampDM816XTimer_TimerInstance_Timer7:
                          module->timerBaseAdrs = (Ptr)TimestampDM816XTimer_TimerBaseAdrs_ARM_Timer7;
                          break;
                  default:
                          break;
                  }
#endif

          } else {
                  module->timerBaseAdrs = TimestampDM816XTimer_timerBaseAdrs;
          }

    if (TimestampDM816XTimer_autoStart){
                  if (module->timerBaseAdrs != NULL){
                          timer = (TimerRegs *)module->timerBaseAdrs;
                          if (TimestampDM816XTimer_prcmClkMuxBaseAdrs == NULL){
#ifndef xdc_target__isaCompatible_64P
                                  switch(TimestampDM816XTimer_timerNumber){
                                  case TimestampDM816XTimer_TimerInstance_Timer4:
                                          pPrcmTimerClkCtrlReg = (Int *)TimestampDM816XTimer_PrcmClkMuxBaseAdrs_Timer4;
                                          break;
                                  case TimestampDM816XTimer_TimerInstance_Timer5:
                                          pPrcmTimerClkCtrlReg = (Int *)TimestampDM816XTimer_PrcmClkMuxBaseAdrs_Timer5;
                                          break;
                                  case TimestampDM816XTimer_TimerInstance_Timer6:
                                          pPrcmTimerClkCtrlReg = (Int *)TimestampDM816XTimer_PrcmClkMuxBaseAdrs_Timer6;
                                          break;
                                  case TimestampDM816XTimer_TimerInstance_Timer7:
                                          pPrcmTimerClkCtrlReg = (Int *)TimestampDM816XTimer_PrcmClkMuxBaseAdrs_Timer7;
                                          break;
                                  default:
                                          break;
                                  }
#endif
                          } else {
                                  pPrcmTimerClkCtrlReg = TimestampDM816XTimer_prcmClkMuxBaseAdrs;
                          }

                          key = Gate_enterSystem();
                          if (pPrcmTimerClkCtrlReg != NULL){
                                  *pPrcmTimerClkCtrlReg = TimestampDM816XTimer_prcmClkMuxInitValue;
                          }


                          /* Timer LoaD Register (TLDR) */
                          timer->tldr = 0; //configure for Timer_MAX_PERIOD
                          while (timer->twps & TIMER_TWPS_W_PEND_TLDR)
                                           ;

                          /* Timer OCP Configuration Register (TIOCP_CFG) */
                         timer->tiocpCfg = (TimestampDM816XTimer_tiocpCfg.softreset << 0) |
                                                           (TimestampDM816XTimer_tiocpCfg.emufree  << 1) |
                                                           (TimestampDM816XTimer_tiocpCfg.idlemode  << 2);

                         /* Timer Match Register (TMAR) */
                         timer->tmar = TIMER_TMAR_MATCH_VALUE;
                         while (timer->twps & TIMER_TWPS_W_PEND_TMAR)
                                 ;

                         /* Timer IRQ WAKEUP ENABLE (IRQWAKEEN) */
                         timer->twer = (TimestampDM816XTimer_twer.mat_wup_ena) |
                                                   (TimestampDM816XTimer_twer.ovf_wup_ena << 1) |
                                                   (TimestampDM816XTimer_twer.tcar_wup_ena << 2);

                         /* Timer IRQENABLE CLR (IRQENABLE_CLR) */
                         timer->tier = (TimestampDM816XTimer_tier.mat_it_ena) |
                                                   (TimestampDM816XTimer_tier.ovf_it_ena << 1) |
                                                   (TimestampDM816XTimer_tier.tcar_it_ena << 2);


                         Gate_leaveSystem(key);
                         TimestampDM816XTimer_start();
                 }
    }
    return (Startup_DONE);
}

/*
 *  ======== TimestampDM816XTimer_start ========
 *  1. Hwi_disable();
 *  2. Clear the counters
 *  3. Clear IFR
 *  4. Enable timer interrupt
 *  5. Start timer
 *  6. Hwi_restore()
 */
Void TimestampDM816XTimer_start()
{
    IArg key;
    TimerRegs *timer;

    timer = (TimerRegs *)TimestampDM816XTimer_timerBaseAdrs;

    key = Gate_enterSystem();

    TimestampDM816XTimer_stop();

    /* Timer Counter Register  (TCRR): configure for Timer_MAX_PERIOD */
    timer->tcrr = 0;

    while (timer->twps & TIMER_TWPS_W_PEND_TCRR)
        ;

         /* Start the timer, with auto-refresh enabled */
        timer->tclr =      (TIMER_TCLR_START_CONTINUOUS) |
                                                 (TimestampDM816XTimer_tclr.ptv << 2) |
                                 (TimestampDM816XTimer_tclr.pre << 5) |
                                 (TimestampDM816XTimer_tclr.ce << 6) |
                                 (TimestampDM816XTimer_tclr.scpwm << 7) |
                                 (TimestampDM816XTimer_tclr.tcm << 8) |
                                 (TimestampDM816XTimer_tclr.trg << 10) |
                                 (TimestampDM816XTimer_tclr.pt << 12) |
                                 (TimestampDM816XTimer_tclr.captmode << 13) |
                                 (TimestampDM816XTimer_tclr.gpocfg << 14);

    while (timer->twps & TIMER_TWPS_W_PEND_TCLR)
        ;

    Gate_leaveSystem(key);
}

/*
 *  ======== TimestampDM816XTimer_stop ========
 *  1. stop timer
 *  2. disable timer interrupt
 */
Void TimestampDM816XTimer_stop()
{
    Bits32 tisr;
    TimerRegs *timer;

    timer = (TimerRegs *)module->timerBaseAdrs;

    timer->tclr &= TIMER_TCLR_STOP_MASK;
    while (timer->twps & TIMER_TWPS_W_PEND_TCLR)
        ;


    tisr = timer->tisr;
    if (tisr) {
        timer->tisr = tisr;     /* clear all pending ints */
    }

}
/*
 *  ======== TimestampDM816XTimer_get32 ========
 */
Bits32 TimestampDM816XTimer_get32() {
        if (module->timerBaseAdrs != NULL){
        return ((Bits32)((TimerRegs *)module->timerBaseAdrs)->tcrr);
        } else {
                return 0;
        }
}

/*
 *  ======== TimestampDM816XTimer_get64 ========
 *  Reads the timer's timestamp register and returns the 64b result.
 *  
 *  NOTE: caller's must ensure that this function is protected
 *  from preemption in order to ensure that a coherent 64b result is returned.
 *  Since this is a shared resource, it is best if some form of
 *  system-level lock (e.g. Lampart's bakery algorithm, spinlock or Peterson lock)
 *  or IPC round-robin mechanism is used to ensure that there are no collisions.
 */
Void TimestampDM816XTimer_get64(Types_Timestamp64 *result) {
        if (module->timerBaseAdrs != NULL){
                result->lo = (Bits32)((TimerRegs *)module->timerBaseAdrs)->tcrr;
                result->hi = module->timerMSW;
        } else {
                result->lo = 0;
                result->hi = 0;
        }
}

/*
 *  ======== TimestampDM816XTimer_getFreq ========
 */
Void TimestampDM816XTimer_getFreq(Types_FreqHz *freq) {
    freq->lo = TimestampDM816XTimer_maxTimerClockFreq.lo;
    freq->hi = TimestampDM816XTimer_maxTimerClockFreq.hi;
}

/*
 * ======== TimestampDM816XTimer_setMSW ========
 * Sets the value to be returned in result->hi by the get64 API
 */
Void TimerstampDM816XTimer_setMSW(Int value){
        module->timerMSW = value;
}

/*
 *  @(#) ti.uia.family.dm; 1, 0, 0, 0,6; 6-15-2011 11:41:42; /db/vtree/library/trees/uia/uia.git/src/ null
 */

