/* 
 * Copyright (c) 2011, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * */

/*
 * ======== UIAMetaData.xdc
 */

/*!
 * Unified Arch. Hardware Interrupt Context Instrumentation
 *
 * The UIAHWICtx module defines context change events
 * and methods that allow tooling to identify hardware interrutp context
 * switches and to enable HWI-aware filtering, trace and
 * analysis.
 */

module UIAMetaData {

metaonly function generateXML();

/*!
 *  ======== genXmlEntry ========
 *  write an entry into an xml file
 *
 *  e.g. <myTag>myValue</myTag>\n
 */
metaonly function genXmlEntry(indent, tag, value);
/*!
 *  ======== genXmlEntryOpen ========
 *  write an entry into an xml file with a key field
 *
 *  e.g. <myTag myKeyName=myKeyValue>
 */
metaonly function  genXmlEntryOpen(indent,  tag);

/*!
 *  ======== genXmlEntryOpenWithKey ========
 *  write an entry into an xml file with a key field
 *
 *  e.g. <myTag myKeyName=myKeyValue>
 */
metaonly function  genXmlEntryOpenWithKey(indent,tag,keyName,keyValue,value);

/*!
 *  ======== genXmlEntryWithKey ========
 *  write an entry into an xml file with a key field
 *
 *  e.g. <myTag myKeyName=myKeyValue>myValue</myTag>\n
 */
 metaonly function  genXmlEntryWithKey(indent,tag,keyName,keyValue,value);
 
 /*!
  * ======== genXmlComment ========
  * add an XML style comment to the active xml file
  */
 metaonly function  genXmlComment(indent,  comment);
/*!
 *  ======== genXmlEntryClose ========
 *  write an entry into an xml file with a key field
 *
 *  e.g. </myTag>\n
 */
metaonly function  genXmlEntryClose(indent,  tag);


/*! @_nodoc
 * ======== setLoggingSetupConfigured ========
 * Called by the LoggingSetup module to indicate it is in the .cfg file
 * 
 * Sets an internal metaonly flag that is used to determine the transport used.
 */
metaonly function setLoggingSetupConfigured();

/*! @_nodoc
 * ======== isLoggingSetupConfigured ========
 * Called by other modules to see if the LoggingSetup module is in the .cfg file
 * 
 * Returns true if the LoggingSetup module can be used to determine the transport used.
 */
metaonly function isLoggingSetupConfigured();
}

/*
 */
/*
 *  @(#) ti.uia.runtime; 1, 0, 0, 2,6; 6-15-2011 11:41:52; /db/vtree/library/trees/uia/uia.git/src/ null
 */

